#include "single_qubit_unitary.h"

namespace qstabr {
namespace circuit {

SingleQubitUnitary::SingleQubitUnitary(int qubit)
    : qubit(qubit), alpha(0), beta(0), gamma(0) {}

void SingleQubitUnitary::AddZPhase(const phase::RationalPhase &phase) {
  gamma += phase;
}

void SingleQubitUnitary::AddXPhase(const phase::RationalPhase &phase) {
  if (phase == 0) return;
  if (phase == 1) {
    beta += 1;
    gamma *= -1;
    return;
  }

  if (beta == 0) {
    alpha = alpha + gamma;
    beta = phase;
    gamma = phase::ZERO;
    return;
  }

  if (beta == 1) {
    alpha -= gamma;
    beta = phase + 1;
    gamma = phase::ZERO;
    return;
  }

  alpha -= beta;
  beta = gamma - beta - phase;
  gamma = phase * -1;
}

void SingleQubitUnitary::ClearAlpha() {
  Simplify();
  alpha = phase::ZERO;
}

bool SingleQubitUnitary::CommutesPast(const PauliLetter letter) {
  if (letter == I) return true;
  Simplify();
  if (letter == Y) return alpha == 0 && beta == 0 && gamma == 0;
  if (letter == Z) return beta == 0;
  return alpha == 0 && gamma == 0;
}

bool SingleQubitUnitary::Simplify() {
  if (beta == 1) {
    alpha -= gamma;
    gamma = phase::ZERO;
  }
  if (beta == 0) {
    alpha += gamma;
    gamma = phase::ZERO;
  }
  return alpha == 0 && beta == 0 && gamma == 0;
}

int SingleQubitUnitary::GetQubit() const { return qubit; }

void SingleQubitUnitary::Synthesise(std::ostream &output,
                                    const QubitManager &manager) const {
  if (alpha == 0 && beta == 0 && gamma == 0) return;
  const auto &q = manager.GetIndexQubit(qubit);
  output << "u3(" << beta << "," << (gamma - phase::PI_BY_2) << ","
         << (alpha + phase::PI_BY_2) << ") " << q << ";" << std::endl;
}

}  // namespace circuit
}  // namespace qstabr